clear all; close all; clc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code gives the signal model for the paper: M. Hawes, L. Mihaylova,
% F. Septier and S. Godsill, "Bayesian Compresive 
% Sensing Approaches for Direction of Arrival Estimation with Mutual 
% Coupling Effects," IEEE Transactions on Antennas and Propagation, To
% Appear, 2017.
% If you have questions abou this code then contact m.hawes@sheffield.ac.uk
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Define input variables.
M = 20; % Number of sensors
seperation = .5; % Adjacent antenna separation
deltaN = 1; % Sample the angular range every deltaN degrees
K = 20; % Number of time snap shots
Act_DOA_change = -1;
randomDOA = 1; % 1 to use above as an actual change, 0 to use above value 
               % as a limit placed on random change.
eta = .55; % Used in thresholding technique to find DOA estimates
run = 1; % Number of examples being considered
L = 1; % Number of far field signals impinging on the array
rho1 = .65; % Term used in calculating mutual coupling
rho2 = .25; % Term used in calculating mutual coupling 
phi1 = pi/7; % Term used in calculating mutual coupling
phi2 = pi/10; % Term used in calculating mutual coupling
% Add more rho and phi terms if required.

% Define sensor locations (uniform grid)
d = (0:M-1)*seperation;

% Define angular range
u = 0:deltaN:180;
N = length(u); % N = number of points used to sample the angular range.

% Find steering vector matrix
A = zeros(2*M,2*N);
for n = 1:N
    A(1:M,n) = exp(-1i*2*pi*d(1:M)*cosd(u(n)));
end

% Account for mutual coupling between antennas here (Currently for antennas
% closer than 1.5*\lambda, more terms have to be added for larger 
% separation).
% Initialise mutual coupling matrix with ones along the diagonal.
C = diag(ones(M,1));
% Add the first off-diagonal terms.
for m = 1:M-1
    C(m,m+1) = rho1*exp(-1i*phi1);
    C(m+1,m) = rho1*exp(-1i*phi1);
end
% Add the second off-diagonal terms.
for m = 1:M-2
    C(m,m+2) = rho2*exp(-1i*phi2);
    C(m+2,m) = rho2*exp(-1i*phi2);
end
% Add more loops if required to increase the number of antennas suffering
% from mutual coupling.
% Alter ideal steering vector matrix to realistic steering vector matrix.
A = C*A(1:M,1:N);

% Split steering vector matrix including mutual coupling into real and 
% imaginary parts.
A(1:M,N+1:2*N) = -imag(A(1:M,1:N));
A(M+1:2*M,1:N) = imag(A(1:M,1:N));
A(M+1:2*M,N+1:2*N) = real(A(1:M,1:N));
A(1:M,1:N) = real(A(1:M,1:N));
% Find the actual DOAs and signals
DOA_act = zeros(L,K,run);
x_a = zeros(L,K,run);
for n = 1:run
    % DOAs
    check = 1;
    while check == 1
        %DOA_act(:,1,n) = 1+round(rand*178); % Random DOA starting point.
        DOA_act(:,1,n) = 20; % Fixed DOA starting point.
        if randomDOA
            % Add more loops if different changes in DOA are desired.
            for k = 1:K
                DOA_change(n,1:L,k) = Act_DOA_change;
            end

        else
            for k = 1:K
                DOA_change(n,1:L,k) = round(Act_DOA_change.*...
                    sign(randn(L,1))*rand(L,1));
            end
        end
        for k = 2:K
            % Find subsequent actual DOAs.
            DOA_act(:,k,n) = DOA_act(:,k-1,n) + DOA_change(n,:,k);
        end
        % Check if in the allowed range of angles.
        if (0 <= DOA_act(:,:,n)) & (DOA_act(:,:,n) <= 180)
            check = 0;
        end
    end
    % Signals
    for l = 1:L
        x_a(l,:,n) = 1; % Fixed signal values.
        %x_a(l,:,n) = sign(randn); % Random signal values.
    end 
end